from functools import wraps
import xml.etree.ElementTree as ET
import xml.dom.minidom as minidom
import re

def validate_response(func):
    @wraps(func)
    def wrapper(response_text, *args, **kwargs):
        if not response_text:
            return None,  "Invalid input: Text must be a non-empty string."
        return func(response_text, *args, **kwargs)
    return wrapper

def handle_exceptions(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except KeyError as e:
            return None, f"Missing key: {e}"
        except Exception as e:
            return None, f"Unexpected error: {e}"
    return wrapper


@handle_exceptions
@validate_response
def clean_invalid_tags(xml_string):
    def sanitize_tag(match):
        tag = match.group(2)
        safe_tag = re.sub(r'[^a-zA-Z0-9_:.-]', '_', tag) 
        return f"<{match.group(1)}{safe_tag}>"
    return re.sub(r'<(/?)([^<>]+)>', sanitize_tag, xml_string), None


@handle_exceptions
@validate_response
def pretty_print_xml(xml_string):
    clean_xml, error = clean_invalid_tags(xml_string)
    if error:
        return None, f"Invalid tags not handled: {error}"
    dom = minidom.parseString(clean_xml)
    return dom.toprettyxml(indent="  "), None


@handle_exceptions
@validate_response
def create_xml_from_text(text):
    root = ET.Element("DocType")

    pattern = re.compile(r'\*\*(.+?):\*\*\s*') 

    matches = list(pattern.finditer(text))

    if not matches:
        return ET.tostring(root, encoding="unicode", method="xml"), None

    for i, match in enumerate(matches):
        section_name = match.group(1).strip().replace(" ", "")
        section_elem = ET.SubElement(root, section_name)

        start = match.end()
        end = matches[i + 1].start() if i + 1 < len(matches) else len(text)
        content = text[start:end].strip()

        if re.search(r'^\s*[\*\d]+\.', content, re.MULTILINE):
            lines = re.split(r'\n\s*[\*\d]+\.*\s*', content)
        elif '*' in content:
            lines = re.split(r'\n\s*\*\s*', content)
        else:
            lines = [content]

        for line in lines:
            line = line.strip()
            if not line:
                continue

            inline_items = re.findall(r'\*\*(.+?)\*\*:\s*(.+)', line)
            if inline_items:
                item_elem = ET.SubElement(section_elem, "Item")
                for key, value in inline_items:
                    sub_elem = ET.SubElement(item_elem, key.strip().replace(" ", ""))
                    sub_elem.text = value.strip()
            else:
                item_elem = ET.SubElement(section_elem, "Item")
                item_elem.text = line

    raw_xml = ET.tostring(root, encoding="unicode", method="xml")
    formatted_xml, parsing_error = pretty_print_xml(raw_xml)
    return formatted_xml, parsing_error

